/*
 * Filesystem object provider support.
 *
 * Copyright (C) 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/accountable.h>
#include <fsserver/notification.h>
#include <fsserver/provider_registry.h>
#include <resource/resource.h>



/* A provider offering notification support and accounting capabilities. */

class Provider : public NotificationSupport, public Accountable
{
protected:
    fileid_t _fileid;
    ProviderRegistry *_registry;
    bool _remove = false;

public:
    explicit Provider(fileid_t fileid, ProviderRegistry *registry);

    virtual ~Provider();

    virtual ProviderRegistry *registry();

    virtual long make_resource(flags_t flags, offset_t *size,
                               object_flags_t *object_flags,
                               Resource **resource) = 0;

    /* Lifecycle methods. */

    virtual bool removal_pending();

    virtual void remove_pending(bool remove);
};

// vim: tabstop=4 expandtab shiftwidth=4
